@@ -70,15 +70,16 @@ def has_audio_stream(video_path: str) -> bool:
 
 class MasterScene:
     """マスターシーンクラス - 全体の動画を管理"""
-    def __init__(self, width: int = 1920, height: int = 1080, fps: int = 60, quality: Literal["low", "medium", "high"] = "medium"):
+    def __init__(self, output_filename: str = "output_video.mp4", width: int = 1920, height: int = 1080, fps: int = 60, quality: Literal["low", "medium", "high"] = "medium"):
         self.width = width
         self.height = height
         self.fps = fps
         self.quality = quality  # "low", "medium", "high"
         self.scenes: List[Scene] = []
         self.total_duration = 0.0
-        self.output_filename = "output_video.mp4"
+        self.output_filename = output_filename
         self.audio_elements = []  # オーディオ要素を追跡
+        self._has_content_at_start = False  # Track if master scene has content at time 0
         
         # 品質設定に基づいてスーパーサンプリング倍率を設定
         self.quality_multipliers = {
@@ -90,9 +91,38 @@ class MasterScene:
         self.render_width = width * self.render_scale
         self.render_height = height * self.render_scale
     
-    def add(self, scene: Scene):
-        """シーンを追加"""
-        self.scenes.append(scene)
+    def add(self, scene: Scene, layer: Literal["top", "bottom"] = "top"):
+        """シーンを追加
+        
+        Args:
+            scene: 追加するシーン
+            layer: "top" を指定すると一番上に追加（最後にレンダリング）、"bottom" を指定すると一番下に追加（最初にレンダリング）
+        """
+        # シーンのstart_timeが明示的に設定されていない場合（Noneの場合）、
+        # 前のシーンの終了時間を開始時間として設定（逐次再生）
+        if scene.start_time is None:
+            # シーンが0秒時点でコンテンツを持っているかチェック
+            scene_has_content_at_start = self._scene_has_content_at_time(scene, 0.0)
+            
+            if scene_has_content_at_start and not self._has_content_at_start:
+                # シーンに0秒時点でコンテンツがあり、マスターシーンにまだ0秒コンテンツがない場合
+                # シーンを0秒から開始させる
+                scene.start_time = 0.0
+                self._has_content_at_start = True
+            elif self.scenes:
+                # 通常の逐次配置
+                last_scene = self.scenes[-1]
+                last_scene_start = last_scene.start_time if last_scene.start_time is not None else 0.0
+                scene.start_time = last_scene_start + last_scene.duration
+            else:
+                # 最初のシーンでstart_timeが設定されていない場合は0から開始
+                scene.start_time = 0.0
+        
+        # Add scene based on layer parameter
+        if layer == "bottom":
+            self.scenes.insert(0, scene)
+        else:  # layer == "top" (default)
+            self.scenes.append(scene)
         # 全体の継続時間を更新
         scene_end_time = scene.start_time + scene.duration
         self.total_duration = max(self.total_duration, scene_end_time)
@@ -104,6 +134,29 @@ class MasterScene:
         self._update_master_bgm_durations()
         return self
     
+    def _scene_has_content_at_time(self, scene, time: float) -> bool:
+        """Check if a scene has any visible content at the specified time.
+        
+        Args:
+            scene: Scene to check
+            time: Time to check (relative to scene start)
+            
+        Returns:
+            True if scene has visible content at the specified time
+        """
+        for element in scene.elements:
+            if isinstance(element, Scene):
+                # Recursively check nested scenes
+                element_start = element.start_time if element.start_time is not None else 0.0
+                if element_start <= time < element_start + element.duration:
+                    if self._scene_has_content_at_time(element, time - element_start):
+                        return True
+            else:
+                # Check if non-scene element is visible at this time
+                if element.start_time <= time < element.start_time + element.duration:
+                    return True
+        return False
+    
     def _update_master_bgm_durations(self):
         """マスターシーン全体の長さに合わせてBGMの持続時間を更新"""
         from .audio_element import AudioElement
@@ -113,19 +166,35 @@ class MasterScene:
                 if self.total_duration > audio_element.duration:
                     audio_element.duration = self.total_duration
     
-    def _collect_audio_elements(self, scene: Scene):
-        """シーンからオーディオ要素を収集"""
+    def _collect_audio_elements(self, scene: Scene, time_offset: float = 0.0):
+        """シーンからオーディオ要素を収集（ネストされたシーンにも対応）
+        
+        Args:
+            scene: Audio elements to collect from
+            time_offset: Cumulative time offset from parent scenes
+        """
         from .audio_element import AudioElement
         from .video_element import VideoElement
+        
+        # Calculate the total time offset for this scene
+        scene_start = scene.start_time if scene.start_time is not None else 0.0
+        total_offset = time_offset + scene_start
+        
         for element in scene.elements:
-            if isinstance(element, AudioElement):
-                # Standalone audio elements are always valid
+            if isinstance(element, Scene):
+                # Recursively collect from nested scenes
+                self._collect_audio_elements(element, total_offset)
+            elif isinstance(element, AudioElement):
+                # Adjust audio timing to account for cumulative scene start times
+                element.start_time += total_offset
                 self.audio_elements.append(element)
             elif isinstance(element, VideoElement):
                 # Ensure the video element's audio element is created
                 element._ensure_audio_element()
                 audio_element = element.get_audio_element()
                 if audio_element is not None:
+                    # Adjust audio timing to account for cumulative scene start times
+                    audio_element.start_time += total_offset
                     # Only add if the video actually has audio
                     self.audio_elements.append(audio_element)
     
@@ -157,17 +226,22 @@ class MasterScene:
         return self
     
     def _apply_quality_to_scene(self, scene):
-        """シーンの要素に品質設定を適用"""
+        """シーンの要素に品質設定を適用（ネストされたシーンにも対応）"""
         from .text_element import TextElement
         from .image_element import ImageElement
         from .video_element import VideoElement
         
         for element in scene.elements:
-            if isinstance(element, TextElement):
+            if isinstance(element, Scene):
+                # Recursively apply quality to nested scenes
+                self._apply_quality_to_scene(element)
+            elif isinstance(element, TextElement):
                 if not hasattr(element, 'quality_scale') or element.quality_scale != self.render_scale:
                     element.quality_scale = self.render_scale
                     # テクスチャを再作成するためのフラグをリセット
                     element.texture_created = False
+                    # サイズも再計算する
+                    element.calculate_size()
             # TODO: ImageElementとVideoElementも同様に対応
     
     def _init_opengl(self):
@@ -192,19 +266,14 @@ class MasterScene:
     
     def _setup_video_writer(self):
         """動画書き込み設定"""
-        # 出力ディレクトリを作成
-        output_dir = "output"
-        if not os.path.exists(output_dir):
-            os.makedirs(output_dir)
-        
         # オーディオ要素がある場合は一時ファイル、ない場合は指定ファイル名
         if self.audio_elements:
             # 一時ファイル名を作成
             base_name = os.path.splitext(self.output_filename)[0]
-            temp_filename = f"{base_name}_temp_video_only.mp4"
-            full_path = os.path.join(output_dir, temp_filename)
+            ext = os.path.splitext(self.output_filename)[1]
+            full_path = f"{base_name}_temp_video_only{ext}"
         else:
-            full_path = os.path.join(output_dir, self.output_filename)
+            full_path = self.output_filename
         
         fourcc = cv2.VideoWriter_fourcc(*'mp4v')
         
@@ -217,18 +286,24 @@ class MasterScene:
     
     def _capture_frame(self):
         """現在の画面をキャプチャ"""
-        # 高解像度でキャプチャ
-        pixels = glReadPixels(0, 0, self.render_width, self.render_height, GL_RGB, GL_UNSIGNED_BYTE)
+        # 高解像度でキャプチャ（アルファチャンネル込み）
+        pixels = glReadPixels(0, 0, self.render_width, self.render_height, GL_RGBA, GL_UNSIGNED_BYTE)
         image = np.frombuffer(pixels, dtype=np.uint8)
-        image = image.reshape((self.render_height, self.render_width, 3))
+        image = image.reshape((self.render_height, self.render_width, 4))
         image = np.flipud(image)  # OpenGLは左下が原点なので上下反転
-        image = cv2.cvtColor(image, cv2.COLOR_RGB2BGR)
+        
+        # RGBAからBGRAに変換
+        image = cv2.cvtColor(image, cv2.COLOR_RGBA2BGRA)
         
         # スーパーサンプリングの場合は出力解像度にダウンスケール
         if self.render_scale > 1:
             image = cv2.resize(image, (self.width, self.height), interpolation=cv2.INTER_AREA)
         
-        return image
+        # MP4出力用にアルファチャンネルを除去してBGRに変換
+        # 透明部分は黒背景とブレンドされる
+        bgr_image = cv2.cvtColor(image, cv2.COLOR_BGRA2BGR)
+        
+        return bgr_image
     
     def _create_audio_mix(self, video_path: str):
         """FFmpegを使ってビデオにオーディオを追加"""
@@ -250,8 +325,7 @@ class MasterScene:
             print("  Ubuntu: sudo apt install ffmpeg")
             return video_path
         
-        output_dir = "output"
-        final_output = os.path.join(output_dir, self.output_filename)
+        final_output = self.output_filename
         
         # 複数のオーディオファイルを処理するためのコマンド構築
         cmd = ['ffmpeg', '-y', '-i', video_path]
